<?php
/*
Plugin Name: WooCommerce Category Discount
Description: A plugin that applies a maximum price for products in specific categories when the cart contains more than a specified quantity from that category. Adds custom text on product list and product page.
Version: 2.1
Author: EasyToolsWP
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Prevent direct access
}

// Add a submenu under WooCommerce
add_action( 'admin_menu', 'category_discount_add_woocommerce_menu' );
add_action( 'admin_init', 'category_discount_settings_init' );

function category_discount_add_woocommerce_menu() {
    add_submenu_page(
        'woocommerce',
        'Category Discount', // Page title
        'Category Discount', // Menu title
        'manage_options',    // Capability
        'category_discount', // Menu slug
        'category_discount_options_page' // Callback to display the settings page
    );
}

function category_discount_settings_init() {
    register_setting( 'categoryDiscountSettings', 'category_discount_settings', 'sanitize_category_discount_settings' );

    add_settings_section(
        'category_discount_section', 
        __( 'Category Discount Settings', 'category-discount' ), 
        'category_discount_settings_section_callback', 
        'categoryDiscountSettings'
    );

    // Fields for settings
    add_settings_field( 
        'category_rules', 
        __( 'Discount Rules', 'category-discount' ), 
        'category_rules_render', 
        'categoryDiscountSettings', 
        'category_discount_section' 
    );

    add_settings_field( 
        'prefix_text', 
        __( 'Prefix Text', 'category-discount' ), 
        'prefix_text_render', 
        'categoryDiscountSettings', 
        'category_discount_section' 
    );

    add_settings_field( 
        'for_text', 
        __( '"For" Text', 'category-discount' ), 
        'for_text_render', 
        'categoryDiscountSettings', 
        'category_discount_section' 
    );

    add_settings_field( 
        'suffix_text', 
        __( 'Suffix Text', 'category-discount' ), 
        'suffix_text_render', 
        'categoryDiscountSettings', 
        'category_discount_section' 
    );

    add_settings_field( 
        'in_category_text', 
        __( '"In Category" Text', 'category-discount' ), 
        'in_category_text_render', 
        'categoryDiscountSettings', 
        'category_discount_section' 
    );

    add_settings_field( 
        'show_as_total_amount', 
        __( 'Show as Total Amount', 'category-discount' ), 
        'show_as_total_amount_render', 
        'categoryDiscountSettings', 
        'category_discount_section' 
    );
}

function sanitize_category_discount_settings($input) {
    $sanitized = array();

    foreach ($input as $key => $value) {
        if (is_array($value)) {
            $sanitized[$key] = array(
                'category_id' => intval($value['category_id']),
                'quantity_limit' => intval($value['quantity_limit']),
                'max_price' => floatval($value['max_price'])
            );
        } else {
            if ($key === 'show_as_total_amount') {
                $sanitized[$key] = intval($value); // Checkbox value is 1 or 0
            } else {
                $sanitized[$key] = sanitize_text_field($value);
            }
        }
    }

    return $sanitized;
}

function category_rules_render() {
    $options = get_option( 'category_discount_settings', array() );
    ?>
    <div id="category-rules-container" style="margin-bottom: 20px;">
        <?php if (!empty($options)) : ?>
            <?php foreach ($options as $index => $rule) : ?>
                <?php if (is_array($rule) && isset($rule['category_id'])) : ?>
                    <div class="category-rule" style="margin-bottom: 10px; padding: 10px; border: 1px solid #ddd; background: #f9f9f9;">
                        <input type="number" name="category_discount_settings[<?php echo esc_attr($index); ?>][category_id]" value="<?php echo esc_attr($rule['category_id']); ?>" placeholder="Category ID" style="width: 150px; margin-right: 10px;">
                        <input type="number" name="category_discount_settings[<?php echo esc_attr($index); ?>][quantity_limit]" value="<?php echo esc_attr($rule['quantity_limit']); ?>" placeholder="Quantity Min." style="width: 150px; margin-right: 10px;">
                        <input type="number" step="0.01" name="category_discount_settings[<?php echo esc_attr($index); ?>][max_price]" value="<?php echo esc_attr($rule['max_price']); ?>" placeholder="Max Price" style="width: 150px; margin-right: 10px;">
                        <button type="button" class="remove-rule" style="color: #a00;">Remove</button>
                    </div>
                <?php endif; ?>
            <?php endforeach; ?>
        <?php endif; ?>
    </div>
    <button type="button" id="add-new-rule" class="button button-primary">Add New Rule</button>

    <script type="text/javascript">
        document.addEventListener('DOMContentLoaded', function() {
            const addNewRuleButton = document.getElementById('add-new-rule');
            const rulesContainer = document.getElementById('category-rules-container');

            addNewRuleButton.addEventListener('click', function() {
                const index = rulesContainer.children.length;
                const newRule = document.createElement('div');
                newRule.classList.add('category-rule');
                newRule.style.cssText = 'margin-bottom: 10px; padding: 10px; border: 1px solid #ddd; background: #f9f9f9;';
                newRule.innerHTML = `
                    <input type="number" name="category_discount_settings[${index}][category_id]" placeholder="Category ID" style="width: 150px; margin-right: 10px;">
                    <input type="number" name="category_discount_settings[${index}][quantity_limit]" placeholder="Quantity Min." style="width: 150px; margin-right: 10px;">
                    <input type="number" step="0.01" name="category_discount_settings[${index}][max_price]" placeholder="Max Price" style="width: 150px; margin-right: 10px;">
                    <button type="button" class="remove-rule" style="color: #a00;">Remove</button>
                `;
                rulesContainer.appendChild(newRule);

                newRule.querySelector('.remove-rule').addEventListener('click', function() {
                    rulesContainer.removeChild(newRule);
                });
            });

            rulesContainer.querySelectorAll('.remove-rule').forEach(button => {
                button.addEventListener('click', function() {
                    rulesContainer.removeChild(button.parentElement);
                });
            });
        });
    </script>
    <?php
}

function prefix_text_render() {
    $options = get_option( 'category_discount_settings' );
    ?>
    <input type='text' name='category_discount_settings[prefix_text]' value='<?php echo isset($options['prefix_text']) ? esc_attr($options['prefix_text']) : ''; ?>' placeholder="Prefix Text">
    <?php
}

function for_text_render() {
    $options = get_option( 'category_discount_settings' );
    ?>
    <input type='text' name='category_discount_settings[for_text]' value='<?php echo isset($options['for_text']) ? esc_attr($options['for_text']) : ''; ?>' placeholder='"For" Text'>
    <?php
}

function suffix_text_render() {
    $options = get_option( 'category_discount_settings' );
    ?>
    <input type='text' name='category_discount_settings[suffix_text]' value='<?php echo isset($options['suffix_text']) ? esc_attr($options['suffix_text']) : ''; ?>' placeholder="Suffix Text">
    <?php
}

function in_category_text_render() {
    $options = get_option( 'category_discount_settings' );
    ?>
    <input type='text' name='category_discount_settings[in_category_text]' value='<?php echo isset($options['in_category_text']) ? esc_attr($options['in_category_text']) : 'in category'; ?>' placeholder='"In Category" Text'>
    <?php
}

function show_as_total_amount_render() {
    $options = get_option( 'category_discount_settings' );
    ?>
    <input type='checkbox' name='category_discount_settings[show_as_total_amount]' value='1' <?php checked( 1, isset($options['show_as_total_amount']) ? $options['show_as_total_amount'] : 0 ); ?> > Show total amount instead of per piece
    <?php
}

function category_discount_settings_section_callback() {
    echo __( 'Set discounts for specific categories based on the number of items in the cart. Customize the prefix, "for", and suffix text to display on product pages.', 'category-discount' );
}

function category_discount_options_page() {
    ?>
    <div class="wrap">
        <h2>Category Discount Settings</h2>
        <form action='options.php' method='post'>
            <?php
            settings_fields( 'categoryDiscountSettings' );
            do_settings_sections( 'categoryDiscountSettings' );
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// Hook to modify the cart prices
add_action( 'woocommerce_before_calculate_totals', 'apply_category_discount_multiple' );

function apply_category_discount_multiple( $cart ) {
    if ( is_admin() && ! defined( 'DOING_AJAX' ) ) return;

    // Get the plugin settings
    $options = get_option( 'category_discount_settings', array() );

    // Loop through each category rule and apply the discounts
    foreach ( $options as $rule ) {
        if (is_array($rule) && isset($rule['category_id'])) {
            $category_id = intval( $rule['category_id'] );
            $quantity_limit = intval( $rule['quantity_limit'] );
            $max_price = floatval( $rule['max_price'] );

            if ( $category_id && $quantity_limit && $max_price ) {
                $total_quantity_in_category = 0;

                // Loop through cart items to count products from this category
                foreach ( $cart->get_cart() as $cart_item_key => $cart_item ) {
                    $product_id = $cart_item['product_id'];

                    // Check if the product belongs to the current category
                    if ( has_term( $category_id, 'product_cat', $product_id ) ) {
                        $total_quantity_in_category += $cart_item['quantity'];
                    }
                }

                // Apply discount if the quantity limit is met or exceeded
                if ( $total_quantity_in_category >= $quantity_limit ) {
                    foreach ( $cart->get_cart() as $cart_item_key => $cart_item ) {
                        $product_id = $cart_item['product_id'];

                        // Apply discount to products in the specific category
                        if ( has_term( $category_id, 'product_cat', $product_id ) ) {
                            $price = $cart_item['data']->get_price();

                            // Set the price to the maximum if it's higher
                            if ( $price > $max_price ) {
                                $cart_item['data']->set_price( $max_price );
                            }
                        }
                    }
                }
            }
        }
    }
}

// Add custom text under the category title (H1) and on single product page
add_action( 'woocommerce_archive_description', 'add_discount_text_to_category', 15 );
add_action( 'woocommerce_single_product_summary', 'add_discount_text_to_single_product_under_price', 15 );

function add_discount_text_to_category() {
    if ( is_product_category() ) {
        $category = get_queried_object();
        if ($category) {
            display_discount_text_for_category($category->term_id);
        }
    }
}

function add_discount_text_to_single_product_under_price() {
    global $product;
    $categories = wp_get_post_terms($product->get_id(), 'product_cat', ['fields' => 'ids']);
    if (!empty($categories)) {
        foreach ($categories as $category_id) {
            if (display_discount_text_for_category($category_id)) {
                break;
            }
        }
    }
}

function display_discount_text_for_category($category_id) {
    $options = get_option( 'category_discount_settings', array() );

    // Retrieve custom texts from settings
    $prefix = isset($options['prefix_text']) ? esc_html($options['prefix_text']) : '';
    $for_text = isset($options['for_text']) ? esc_html($options['for_text']) : 'for';
    $suffix = isset($options['suffix_text']) ? esc_html($options['suffix_text']) : '';
    $in_category_text = isset($options['in_category_text']) ? esc_html($options['in_category_text']) : 'in category';
    $show_as_total = isset($options['show_as_total_amount']) ? intval($options['show_as_total_amount']) : 0;

    foreach ($options as $rule) {
        if (is_array($rule) && intval($rule['category_id']) === intval($category_id)) {
            $quantity = intval($rule['quantity_limit']);
            $max_price = floatval($rule['max_price']);
            $price_display = $show_as_total ? wc_price($max_price * $quantity) : wc_price($max_price);

            // Get the category name and link
            $category_link = get_term_link($category_id, 'product_cat');
            $category_name = get_term($category_id, 'product_cat')->name;

            $category_text = sprintf('%s <a href="%s">%s</a>', $in_category_text, esc_url($category_link), esc_html($category_name));

            // Construct the display text
            $custom_text = sprintf('%s %d %s %s %s %s', $prefix, $quantity, $category_text, $for_text, $price_display, $suffix);
            echo '<p class="custom-discount-text">' . $custom_text . '</p>';
            return true; // Stop after the first matching rule
        }
    }
    return false;
}
